@@ -71,6 +71,7 @@
 #include <QCloseEvent>
 #include <QMessageBox>
 #include <QMimeData>
+#include <QMimeDatabase>
 #if defined(QT_PRINTSUPPORT_LIB)
 #include <QtPrintSupport/qtprintsupportglobal.h>
 #if QT_CONFIG(printer)
@@ -141,13 +142,23 @@ TextEdit::TextEdit(QWidget *parent)
 
 #ifndef QT_NO_CLIPBOARD
     actionCut->setEnabled(false);
+    connect(textEdit, &QTextEdit::copyAvailable, actionCut, &QAction::setEnabled);
     actionCopy->setEnabled(false);
+    connect(textEdit, &QTextEdit::copyAvailable, actionCopy, &QAction::setEnabled);
 
     connect(QApplication::clipboard(), &QClipboard::dataChanged, this, &TextEdit::clipboardDataChanged);
 #endif
 
     textEdit->setFocus();
     setCurrentFileName(QString());
+
+#ifdef Q_OS_MACOS
+    // Use dark text on light background on macOS, also in dark mode.
+    QPalette pal = textEdit->palette();
+    pal.setColor(QPalette::Base, QColor(Qt::white));
+    pal.setColor(QPalette::Text, QColor(Qt::black));
+    textEdit->setPalette(pal);
+#endif
 }
 
 void TextEdit::closeEvent(QCloseEvent *e)
@@ -306,6 +317,14 @@ void TextEdit::setupTextActions()
     actionAlignJustify->setShortcut(Qt::CTRL + Qt::Key_J);
     actionAlignJustify->setCheckable(true);
     actionAlignJustify->setPriority(QAction::LowPriority);
+    const QIcon indentMoreIcon = QIcon::fromTheme("format-indent-more", QIcon(rsrcPath + "/format-indent-more.png"));
+    actionIndentMore = menu->addAction(indentMoreIcon, tr("&Indent"), this, &TextEdit::indent);
+    actionIndentMore->setShortcut(Qt::CTRL + Qt::Key_BracketRight);
+    actionIndentMore->setPriority(QAction::LowPriority);
+    const QIcon indentLessIcon = QIcon::fromTheme("format-indent-less", QIcon(rsrcPath + "/format-indent-less.png"));
+    actionIndentLess = menu->addAction(indentLessIcon, tr("&Unindent"), this, &TextEdit::unindent);
+    actionIndentLess->setShortcut(Qt::CTRL + Qt::Key_BracketLeft);
+    actionIndentLess->setPriority(QAction::LowPriority);
 
     // Make sure the alignLeft  is always left of the alignRight
     QActionGroup *alignGroup = new QActionGroup(this);
@@ -324,6 +343,10 @@ void TextEdit::setupTextActions()
 
     tb->addActions(alignGroup->actions());
     menu->addActions(alignGroup->actions());
+    tb->addAction(actionIndentMore);
+    tb->addAction(actionIndentLess);
+    menu->addAction(actionIndentMore);
+    menu->addAction(actionIndentLess);
 
     menu->addSeparator();
 
@@ -332,6 +355,15 @@ void TextEdit::setupTextActions()
     actionTextColor = menu->addAction(pix, tr("&Color..."), this, &TextEdit::textColor);
     tb->addAction(actionTextColor);
 
+    menu->addSeparator();
+
+    const QIcon checkboxIcon = QIcon::fromTheme("status-checkbox-checked", QIcon(rsrcPath + "/checkbox-checked.png"));
+    actionToggleCheckState = menu->addAction(checkboxIcon, tr("Chec&ked"), this, &TextEdit::setChecked);
+    actionToggleCheckState->setShortcut(Qt::CTRL + Qt::Key_K);
+    actionToggleCheckState->setCheckable(true);
+    actionToggleCheckState->setPriority(QAction::LowPriority);
+    tb->addAction(actionToggleCheckState);
+
     tb = addToolBar(tr("Format Actions"));
     tb->setAllowedAreas(Qt::TopToolBarArea | Qt::BottomToolBarArea);
     addToolBarBreak(Qt::TopToolBarArea);
@@ -343,17 +375,25 @@ void TextEdit::setupTextActions()
     comboStyle->addItem("Bullet List (Disc)");
     comboStyle->addItem("Bullet List (Circle)");
     comboStyle->addItem("Bullet List (Square)");
+    comboStyle->addItem("Task List (Unchecked)");
+    comboStyle->addItem("Task List (Checked)");
     comboStyle->addItem("Ordered List (Decimal)");
     comboStyle->addItem("Ordered List (Alpha lower)");
     comboStyle->addItem("Ordered List (Alpha upper)");
     comboStyle->addItem("Ordered List (Roman lower)");
     comboStyle->addItem("Ordered List (Roman upper)");
+    comboStyle->addItem("Heading 1");
+    comboStyle->addItem("Heading 2");
+    comboStyle->addItem("Heading 3");
+    comboStyle->addItem("Heading 4");
+    comboStyle->addItem("Heading 5");
+    comboStyle->addItem("Heading 6");
 
     connect(comboStyle, QOverload<int>::of(&QComboBox::activated), this, &TextEdit::textStyle);
 
     comboFont = new QFontComboBox(tb);
     tb->addWidget(comboFont);
-    connect(comboFont, QOverload<const QString &>::of(&QComboBox::activated), this, &TextEdit::textFamily);
+    connect(comboFont, &QComboBox::textActivated, this, &TextEdit::textFamily);
 
     comboSize = new QComboBox(tb);
     comboSize->setObjectName("comboSize");
@@ -361,11 +401,11 @@ void TextEdit::setupTextActions()
     comboSize->setEditable(true);
 
     const QList<int> standardSizes = QFontDatabase::standardSizes();
-    foreach (int size, standardSizes)
+    for (int size : standardSizes)
         comboSize->addItem(QString::number(size));
     comboSize->setCurrentIndex(standardSizes.indexOf(QApplication::font().pointSize()));
 
-    connect(comboSize, QOverload<const QString &>::of(&QComboBox::activated), this, &TextEdit::textSize);
+    connect(comboSize, &QComboBox::textActivated, this, &TextEdit::textSize);
 }
 
 bool TextEdit::load(const QString &f)
@@ -379,11 +419,18 @@ bool TextEdit::load(const QString &f)
     QByteArray data = file.readAll();
     QTextCodec *codec = Qt::codecForHtml(data);
     QString str = codec->toUnicode(data);
+    QUrl baseUrl = (f.front() == QLatin1Char(':') ? QUrl(f) : QUrl::fromLocalFile(f)).adjusted(QUrl::RemoveFilename);
+    textEdit->document()->setBaseUrl(baseUrl);
     if (Qt::mightBeRichText(str)) {
         textEdit->setHtml(str);
     } else {
-        str = QString::fromLocal8Bit(data);
-        textEdit->setPlainText(str);
+#if QT_CONFIG(textmarkdownreader)
+        QMimeDatabase db;
+        if (db.mimeTypeForFileNameAndData(f, data).name() == QLatin1String("text/markdown"))
+            textEdit->setMarkdown(str);
+        else
+#endif
+            textEdit->setPlainText(QString::fromLocal8Bit(data));
     }
 
     setCurrentFileName(f);
@@ -435,7 +482,15 @@ void TextEdit::fileOpen()
     QFileDialog fileDialog(this, tr("Open File..."));
     fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
     fileDialog.setFileMode(QFileDialog::ExistingFile);
-    fileDialog.setMimeTypeFilters(QStringList() << "text/html" << "text/plain");
+    fileDialog.setMimeTypeFilters(QStringList()
+#if QT_CONFIG(texthtmlparser)
+                                  << "text/html"
+#endif
+#if QT_CONFIG(textmarkdownreader)
+
+                                  << "text/markdown"
+#endif
+                                  << "text/plain");
     if (fileDialog.exec() != QDialog::Accepted)
         return;
     const QString fn = fileDialog.selectedFiles().first();
@@ -469,9 +524,18 @@ bool TextEdit::fileSaveAs()
     QFileDialog fileDialog(this, tr("Save as..."));
     fileDialog.setAcceptMode(QFileDialog::AcceptSave);
     QStringList mimeTypes;
-    mimeTypes << "application/vnd.oasis.opendocument.text" << "text/html" << "text/plain";
+    mimeTypes << "text/plain"
+#if QT_CONFIG(textodfwriter)
+              << "application/vnd.oasis.opendocument.text"
+#endif
+#if QT_CONFIG(textmarkdownwriter)
+              << "text/markdown"
+#endif
+              << "text/html";
     fileDialog.setMimeTypeFilters(mimeTypes);
+#if QT_CONFIG(textodfwriter)
     fileDialog.setDefaultSuffix("odt");
+#endif
     if (fileDialog.exec() != QDialog::Accepted)
         return false;
     const QString fn = fileDialog.selectedFiles().first();
@@ -575,44 +639,73 @@ void TextEdit::textSize(const QString &p)
 void TextEdit::textStyle(int styleIndex)
 {
     QTextCursor cursor = textEdit->textCursor();
+    QTextListFormat::Style style = QTextListFormat::ListStyleUndefined;
+    QTextBlockFormat::MarkerType marker = QTextBlockFormat::MarkerType::NoMarker;
 
-    if (styleIndex != 0) {
-        QTextListFormat::Style style = QTextListFormat::ListDisc;
+    switch (styleIndex) {
+    case 1:
+        style = QTextListFormat::ListDisc;
+        break;
+    case 2:
+        style = QTextListFormat::ListCircle;
+        break;
+    case 3:
+        style = QTextListFormat::ListSquare;
+        break;
+    case 4:
+        if (cursor.currentList())
+            style = cursor.currentList()->format().style();
+        else
+            style = QTextListFormat::ListDisc;
+        marker = QTextBlockFormat::MarkerType::Unchecked;
+        break;
+    case 5:
+        if (cursor.currentList())
+            style = cursor.currentList()->format().style();
+        else
+            style = QTextListFormat::ListDisc;
+        marker = QTextBlockFormat::MarkerType::Checked;
+        break;
+    case 6:
+        style = QTextListFormat::ListDecimal;
+        break;
+    case 7:
+        style = QTextListFormat::ListLowerAlpha;
+        break;
+    case 8:
+        style = QTextListFormat::ListUpperAlpha;
+        break;
+    case 9:
+        style = QTextListFormat::ListLowerRoman;
+        break;
+    case 10:
+        style = QTextListFormat::ListUpperRoman;
+        break;
+    default:
+        break;
+    }
 
-        switch (styleIndex) {
-            default:
-            case 1:
-                style = QTextListFormat::ListDisc;
-                break;
-            case 2:
-                style = QTextListFormat::ListCircle;
-                break;
-            case 3:
-                style = QTextListFormat::ListSquare;
-                break;
-            case 4:
-                style = QTextListFormat::ListDecimal;
-                break;
-            case 5:
-                style = QTextListFormat::ListLowerAlpha;
-                break;
-            case 6:
-                style = QTextListFormat::ListUpperAlpha;
-                break;
-            case 7:
-                style = QTextListFormat::ListLowerRoman;
-                break;
-            case 8:
-                style = QTextListFormat::ListUpperRoman;
-                break;
-        }
+    cursor.beginEditBlock();
 
-        cursor.beginEditBlock();
+    QTextBlockFormat blockFmt = cursor.blockFormat();
 
-        QTextBlockFormat blockFmt = cursor.blockFormat();
+    if (style == QTextListFormat::ListStyleUndefined) {
+        blockFmt.setObjectIndex(-1);
+        int headingLevel = styleIndex >= 11 ? styleIndex - 11 + 1 : 0; // H1 to H6, or Standard
+        blockFmt.setHeadingLevel(headingLevel);
+        cursor.setBlockFormat(blockFmt);
 
+        int sizeAdjustment = headingLevel ? 4 - headingLevel : 0; // H1 to H6: +3 to -2
+        QTextCharFormat fmt;
+        fmt.setFontWeight(headingLevel ? QFont::Bold : QFont::Normal);
+        fmt.setProperty(QTextFormat::FontSizeAdjustment, sizeAdjustment);
+        cursor.select(QTextCursor::LineUnderCursor);
+        cursor.mergeCharFormat(fmt);
+        textEdit->mergeCurrentCharFormat(fmt);
+    } else {
+        blockFmt.setMarker(marker);
+        cursor.setBlockFormat(blockFmt);
         QTextListFormat listFmt;
-
         if (cursor.currentList()) {
             listFmt = cursor.currentList()->format();
         } else {
@@ -620,18 +713,11 @@ void TextEdit::textStyle(int styleIndex)
             blockFmt.setIndent(0);
             cursor.setBlockFormat(blockFmt);
         }
-
         listFmt.setStyle(style);
-
         cursor.createList(listFmt);
-
-        cursor.endEditBlock();
-    } else {
-        // ####
-        QTextBlockFormat bfmt;
-        bfmt.setObjectIndex(-1);
-        cursor.mergeBlockFormat(bfmt);
     }
+
+    cursor.endEditBlock();
 }
 
 void TextEdit::textColor()
@@ -657,6 +743,45 @@ void TextEdit::textAlign(QAction *a)
         textEdit->setAlignment(Qt::AlignJustify);
 }
 
+void TextEdit::setChecked(bool checked)
+{
+    textStyle(checked ? 5 : 4);
+}
+
+void TextEdit::indent()
+{
+    modifyIndentation(1);
+}
+
+void TextEdit::unindent()
+{
+    modifyIndentation(-1);
+}
+
+void TextEdit::modifyIndentation(int amount)
+{
+    QTextCursor cursor = textEdit->textCursor();
+    cursor.beginEditBlock();
+    if (cursor.currentList()) {
+        QTextListFormat listFmt = cursor.currentList()->format();
+        // See whether the line above is the list we want to move this item into,
+        // or whether we need a new list.
+        QTextCursor above(cursor);
+        above.movePosition(QTextCursor::Up);
+        if (above.currentList() && listFmt.indent() + amount == above.currentList()->format().indent()) {
+            above.currentList()->add(cursor.block());
+        } else {
+            listFmt.setIndent(listFmt.indent() + amount);
+            cursor.createList(listFmt);
+        }
+    } else {
+        QTextBlockFormat blockFmt = cursor.blockFormat();
+        blockFmt.setIndent(blockFmt.indent() + amount);
+        cursor.setBlockFormat(blockFmt);
+    }
+    cursor.endEditBlock();
+}
+
 void TextEdit::currentCharFormatChanged(const QTextCharFormat &format)
 {
     fontChanged(format.font());
@@ -666,6 +791,54 @@ void TextEdit::currentCharFormatChanged(const QTextCharFormat &format)
 void TextEdit::cursorPositionChanged()
 {
     alignmentChanged(textEdit->alignment());
+    QTextList *list = textEdit->textCursor().currentList();
+    if (list) {
+        switch (list->format().style()) {
+        case QTextListFormat::ListDisc:
+            comboStyle->setCurrentIndex(1);
+            break;
+        case QTextListFormat::ListCircle:
+            comboStyle->setCurrentIndex(2);
+            break;
+        case QTextListFormat::ListSquare:
+            comboStyle->setCurrentIndex(3);
+            break;
+        case QTextListFormat::ListDecimal:
+            comboStyle->setCurrentIndex(6);
+            break;
+        case QTextListFormat::ListLowerAlpha:
+            comboStyle->setCurrentIndex(7);
+            break;
+        case QTextListFormat::ListUpperAlpha:
+            comboStyle->setCurrentIndex(8);
+            break;
+        case QTextListFormat::ListLowerRoman:
+            comboStyle->setCurrentIndex(9);
+            break;
+        case QTextListFormat::ListUpperRoman:
+            comboStyle->setCurrentIndex(10);
+            break;
+        default:
+            comboStyle->setCurrentIndex(-1);
+            break;
+        }
+        switch (textEdit->textCursor().block().blockFormat().marker()) {
+        case QTextBlockFormat::MarkerType::NoMarker:
+            actionToggleCheckState->setChecked(false);
+            break;
+        case QTextBlockFormat::MarkerType::Unchecked:
+            comboStyle->setCurrentIndex(4);
+            actionToggleCheckState->setChecked(false);
+            break;
+        case QTextBlockFormat::MarkerType::Checked:
+            comboStyle->setCurrentIndex(5);
+            actionToggleCheckState->setChecked(true);
+            break;
+        }
+    } else {
+        int headingLevel = textEdit->textCursor().blockFormat().headingLevel();
+        comboStyle->setCurrentIndex(headingLevel ? headingLevel + 10 : 0);
+    }
 }
 
 void TextEdit::clipboardDataChanged()
