@@ -70,17 +70,17 @@ class RegularExpressionDialog : public QDialog
     Q_OBJECT
 
 public:
-    RegularExpressionDialog(QWidget *parent = 0);
+    RegularExpressionDialog(QWidget *parent = nullptr);
 
 private:
     void refresh();
-    void copyEscapedPatternToClipboard();
     void setupUi();
     QWidget *setupLeftUi();
     QWidget *setupRightUi();
     void setResultUiEnabled(bool enabled);
 
     QLineEdit *patternLineEdit;
+    QLineEdit *rawStringLiteralLineEdit;
     QLineEdit *escapedPatternLineEdit;
 
     QPlainTextEdit *subjectTextEdit;
